﻿--[[
SimpleMD
Author: Michael Joseph Murray aka Starinnia of Lothar(US) aka Lyte of Lothar(US)
Please see license.txt for details.
$Revision: 130 $
$Date: 2009-10-06 17:29:23 +0000 (Tue, 06 Oct 2009) $
Project Version: r130
contact: codemaster2010 AT gmail DOT com

Copyright (c) 2007-2009 Michael J. Murray aka Starinnia of Lothar
All rights reserved unless otherwise explicitly stated.

]]

local ver, rev = "3.0", tonumber(("$Revision: 130 $"):match("%d+"))
local pname = UnitName("player")
local addonID = "SimpleMD-" .. ver .. " r" .. rev 

SimpleMD = LibStub("AceAddon-3.0"):NewAddon("SimpleMD", "AceConsole-3.0", "AceComm-3.0", "AceTimer-3.0", "AceEvent-3.0", "LibSink-2.0")
local LSM = LibStub("LibSharedMedia-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("SimpleMD")

local strformat = string.format
local select = select
--this allows us to disable bars/broadcasts in pvp settings where braodcasting can spam "You aren't in a party."
local allowedZones = {["pvp"] = false, ["arena"] = false, ["raid"] = true, ["party"] = true}

function SimpleMD:OnInitialize()
	local defaults = {
		profile = {
			tenCD = L["%s has 10 seconds left on MD cooldown!"],
			fiveCD = L["%s has 5 seconds left on MD cooldown!"],
			fifteenCD = L["%s has 15 seconds left on MD cooldown!"],
			noCD = L["%s has MD ready!"],
			readiness = L["%s used Readiness! MD ready!"],
			r = 1.0,
			g = 1.0,
			b = 1.0,
			channel = "",
			showInChannel = false,
			showInChat = false,
			showOnScreen = true,
			showFloating = false,
			showSelf = true,
			showRW = false,
			showParty = false,
			showRaid = false,
			showYell = false,
			showBars = true,
			showGain = true,
			showFade = true,
			ready = true,
			ten = true,
			five = false,
			fifteen = false,
			showReadiness = true,
			texture = "Blizzard",
			font = "Friz Quadrata TT",
			barR = 0,
			barG = 0.4,
			barB = 0.08,
			cbarR = 1.0,
			cbarG = 0,
			cbarB = 0,
			BGr = 0,
			BGg = 1.0,
			BGb = 0.04,
			txtR = 1.0,
			txtG = 1.0,
			txtB = 1.0,
			bg_alpha = 1.0,
			textSize = 11,
			barScale = 1.0,
			barWidth = 195.00,
			barHeight = 17.00,
			growUp = false,
			reverse = false,
			stay = false,
			position = {},
			sinkOpts = {},
		}
	}

	self.db = LibStub("AceDB-3.0"):New("SimplemdDB", defaults, "Default")
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Refresh")
	self.db.RegisterCallback(self, "OnProfileChanged", "Refresh")
	self:SetSinkStorage(self.db.profile.sinkOpts)
	
	self.anchor, self.frame = self:CreateAnchor(addonID)
	self.anchor.timers = {}
	self.timercount = 0
	self.recyclePile = {}
	self.idents = {}
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("SimpleMD", self:GetOptions())
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("SimpleMD", "SimpleMD")
	self:RegisterChatCommand("smd", "OpenConfig", true, true)
	self:RegisterChatCommand("simplemd", "OpenConfig")
end

function SimpleMD:OnEnable()
	self.tex = [[Interface\Icons\Ability_Hunter_Misdirection]]
	self:RegisterComm("SimpleMD", "OnCommReceive")
	self:RegisterEvent("RAID_ROSTER_UPDATE", "RaidStatus") --to clear static bars after leaving a raid
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLogEvent")
end

function SimpleMD:Reset()
	self.anchor:ClearAllPoints()
	self.frame:ClearAllPoints()
	self.anchor:SetPoint("CENTER", UIParent, "CENTER")
	self.frame:SetPoint("CENTER", UIParent, "CENTER")
end

function SimpleMD:Refresh()
	local db = self.db.profile
	self.anchor:ClearAllPoints()
	self.frame:ClearAllPoints()
	if db.position.x then
		self.anchor:SetPoint(db.position.point, UIParent, db.position.anchor, db.position.x, db.position.y)
		self.frame:SetPoint(db.position.point, UIParent, db.position.anchor, db.position.x, db.position.y)
	else
		self.anchor:SetPoint("CENTER", UIParent, "CENTER")
		self.frame:SetPoint("CENTER", UIParent, "CENTER")
	end
end

function SimpleMD:OpenConfig()
	LibStub("AceConfigDialog-3.0"):SetDefaultSize("SimpleMD", 500, 550)
	LibStub("AceConfigDialog-3.0"):Open("SimpleMD")
end

function SimpleMD:OnCommReceive(prefix, message, distro, sender)
	if sender ~= pname then
		if message == "SMDREADINESS" then
			if self.db.profile.showReadiness then
				self:PrintMessage(strformat(self.db.profile.readiness, sender), false)
			end
			if self.db.profile.showBars then
				self:ReadinessCheck(sender)
			end
		elseif message == "SMDFIVE" and self.db.profile.five then
			self:PrintMessage(strformat(self.db.profile.fiveCD, sender), false)
		elseif message == "SMDTEN" and self.db.profile.ten then
			self:PrintMessage(strformat(self.db.profile.tenCD, sender), false)
		elseif message == "SMDFIFTEEN" and self.db.profile.fifteen then
			self:PrintMessage(strformat(self.db.profile.fifteenCD, sender), false)
		elseif message == "SMDREADY" and self.db.profile.ready then
			self:PrintMessage(strformat(self.db.profile.noCD, sender), false)
		end
	end
end

function SimpleMD:PrintMessage(message, isPlayer)
	--if isPlayer is true the message regards the player and the message needs to
	--be sent to any selected channels, if it is false then the message is about another hunter
	--and should only be displayed locally to the player
	
	if isPlayer then
		--print to chat frame
		if self.db.profile.showInChat and self.db.profile.showSelf then
			self:Print(message)
		end
		--print to Error Message frame
		if self.db.profile.showOnScreen and self.db.profile.showSelf then
			UIErrorsFrame:AddMessage(message, self.db.profile.r, self.db.profile.g, self.db.profile.b, 5.0)
		end
		--print to Party chat
		if self.db.profile.showParty and GetNumRaidMembers() == 0 then
			SendChatMessage(message, "PARTY")
		end
		--print to Raid Chat
		if self.db.profile.showRaid and UnitInRaid("player") then
			SendChatMessage(message, "RAID")
		end
		--print to Raid Warning
		if (IsRaidLeader() or IsRaidOfficer()) and self.db.profile.showRW then
			SendChatMessage(message, "RAID_WARNING")
		end
		-- print to custom channel
		if self.db.profile.showInChannel and self.db.profile.channel ~= "" then
			SendChatMessage(message, "CHANNEL", nil, GetChannelName(self.db.profile.channel))
		end
		--print to Scrolling Text
		if self.db.profile.showFloating and self.db.profile.showSelf then
			self:Pour(message, self.db.profile.r, self.db.profile.g, self.db.profile.b)
		end
	else
		--print to Scrolling Text mod
		if self.db.profile.showFloating then
			self:Pour(message, self.db.profile.r, self.db.profile.g, self.db.profile.b)
		end
		--print to chat frame
		if self.db.profile.showInChat then
			self:Print(message)
		end
		--print to error frame
		if self.db.profile.showOnScreen then
			UIErrorsFrame:AddMessage(message, self.db.profile.r, self.db.profile.g, self.db.profile.b, 5.0)
		end
	end
end

-- 34477 misdirect on hunter
-- 35079 misdirect gained by target
--23989 readiness
local mdOnCD
function SimpleMD:CombatLogEvent(event, timestamp, subevent, sID, sName, sFlags, dID, dName, dFlags, spellID, spellName)
	if spellID == 34477 then
		if subevent == "SPELL_CAST_SUCCESS" then
			if (GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0) and allowedZones[select(2, IsInInstance())] then
				if self.db.profile.showBars then
					self:CreateTimerBar(sName, dName, 30, self.db.profile.reverse, self.db.profile.stay, self.db.profile.growUp)
				end
				if self.db.profile.showGain then
					self:PrintMessage(strformat(L["%s cast Misdirection on %s"], sName, dName), sName == pname)
				end
				if sName == pname then --if player start CD timers for broadcast
					self:ScheduleTimer("CooldownBroadcast", 15, 15) -- 15 second warning
					self:ScheduleTimer("CooldownBroadcast", 20, 10) -- 10 second warning
					self:ScheduleTimer("CooldownBroadcast", 25, 5) -- 5 second warning
					self:ScheduleTimer("CooldownBroadcast", 30, 0) -- cooldown finished
					mdOnCD = true
				end
			end
		elseif subevent == "SPELL_AURA_REMOVED" then
			if GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0 then
				if self.db.profile.showFade then
					self:PrintMessage(strformat(L["Misdirection fades from %s"], dName), sName == pname)
				end
				--if CD bars are being used, change the color to the normal choice from the special signifying active charges
				if self.db.profile.showBars then
					for k, v in pairs(self.idents) do
						if v == dName then
							self.anchor.timers[k].timer:SetStatusBarColor(self.db.profile.barR, self.db.profile.barG, self.db.profile.barB, 1)
						end
					end
				end
			end
		end
	elseif spellID == 23989 then
	--only deal with readiness if the caster is the player
	--we are alerted of other hunter readiness casts through the comm channel
		if subevent == "SPELL_CAST_SUCCESS" and sName == pname then
			if (GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0) and allowedZones[select(2, IsInInstance())] and mdOnCD then
				self:CancelAllTimers() --cancel all CD message events
				mdOnCD = nil
				if self.db.profile.showReadiness then
					self:PrintMessage(strformat(self.db.profile.readiness, pname), true)
				end
				if UnitInRaid("player") then
					self:SendCommMessage("SimpleMD", "SMDREADINESS", "RAID", nil, "NORMAL")
				else
					self:SendCommMessage("SimpleMD", "SMDREADINESS", "PARTY", nil, "NORMAL")
				end
				if self.db.profile.showSelf and self.db.profile.showBars then
					self:ReadinessCheck(pname)
				end
			end
		end
	end
end

--Function for broadcasting the cooldown messages
--Do nothing if player is dead or ghost since they can't MD anymore (thanks Cerqua)
function SimpleMD:CooldownBroadcast(timeleft)
	if not UnitIsDeadOrGhost("player") and (GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0) then
		if timeleft == 10 then
			if self.db.profile.ten then
				self:PrintMessage(strformat(self.db.profile.tenCD, pname), true)
			end
			if UnitInRaid("player") then
				self:SendCommMessage("SimpleMD", "SMDTEN", "RAID", nil, "NORMAL")
			else
				self:SendCommMessage("SimpleMD", "SMDTEN", "PARTY", nil, "NORMAL")
			end
			return
		elseif timeleft == 15 then
			if self.db.profile.fifteen then
				self:PrintMessage(strformat(self.db.profile.fifteenCD, pname), true)
			end
			if UnitInRaid("player") then
				self:SendCommMessage("SimpleMD", "SMDFIFTEEN", "RAID", nil, "NORMAL")
			else
				self:SendCommMessage("SimpleMD", "SMDFIFTEEN", "PARTY", nil, "NORMAL")
			end
			return
		elseif timeleft == 5 then
			if self.db.profile.five then
				self:PrintMessage(strformat(self.db.profile.fiveCD, pname), true)
			end
			if UnitInRaid("player") then
				self:SendCommMessage("SimpleMD", "SMDFIVE", "RAID", nil, "NORMAL")
			else
				self:SendCommMessage("SimpleMD", "SMDFIVE", "PARTY", nil, "NORMAL")
			end
			return
		elseif timeleft == 0 then
			if self.db.profile.ready then
				self:PrintMessage(strformat(self.db.profile.noCD, pname), true)
			end
			if UnitInRaid("player") then
				self:SendCommMessage("SimpleMD", "SMDREADY", "RAID", nil, "NORMAL")
			else
				self:SendCommMessage("SimpleMD", "SMDREADY", "PARTY", nil, "NORMAL")
			end
			return
		end
	end
end

--see if the player is in a raid, if not.. clear the CD bars
function SimpleMD:RaidStatus()
	if not UnitInRaid("player") then
		self:ClearTimerBars()
	end
end

--Disable the bars and immediately cancel all running CD timers
function SimpleMD:DisableBars()
	if self.db.profile.showBars then
		self.db.profile.showBars = false
	else
		self.db.profile.showBars = true
	end
	
	self:ClearTimerBars()
end

--Test function for all aspects of the mod
function SimpleMD:Test()
	if self.db.profile.showInChat then
		self:Print(L["Testing Chat Frame Printing"])
	end
	if self.db.profile.showOnScreen then
		UIErrorsFrame:AddMessage(L["Testing On Screen Printing"], self.db.profile.r, self.db.profile.g, self.db.profile.b, 5.0)
	end
	if self.db.profile.showInChannel and channel ~= "" then
		SendChatMessage(L["Testing Channel Printing"], "CHANNEL", GetDefaultLanguage("player"), GetChannelName(self.db.profile.channel))
	end
	if self.db.profile.showFloating then
		self:Pour(L["Testing Floating Message"], self.db.profile.r, self.db.profile.g, self.db.profile.b)
	end
	if self.db.profile.showBars then
		self:ClearTimerBars()
		self:CreateTimerBar("Test1", "test_target1", 30, self.db.profile.reverse, self.db.profile.stay, self.db.profile.growUp)
		self:CreateTimerBar("test2", "test_target2", 30, self.db.profile.reverse, self.db.profile.stay, self.db.profile.growUp)
		self:CreateTimerBar("test3", "test_target3", 30, self.db.profile.reverse, self.db.profile.stay, self.db.profile.growUp)
		self:ScheduleTimer("ClearTimerBars", 30)
	end
end
